@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lbjp-common.gss/src/glite_gss.c,v 1.20.2.1 2012/06/13 08:27:24 dkouril Exp $"
+#ident "$Header: /cvs/jra1mw/org.glite.lbjp-common.gss/src/glite_gss.c,v 1.25 2012/08/10 13:23:02 valtri Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -38,7 +38,9 @@ limitations under the License.
 #endif
 
 #include <globus_common.h>
+#ifndef NO_GLOBUS_GSSAPI
 #include <globus_gsi_callback.h>
+#endif
 
 #include <gssapi.h>
 #include <openssl/err.h>
@@ -685,13 +687,21 @@ destroy_proxy(char *proxy_file)
    return 0;
 }
 
+/* XXX for backward compatibility, remove this on next major version bump */
+#undef edg_wll_gss_acquire_cred_gsi
+int
+edg_wll_gss_acquire_cred_gsi(const char *cert_file, const char *key_file, edg_wll_GssCred *cred,
+			 edg_wll_GssStatus* gss_code)
+{
+	return edg_wll_gss_acquire_cred(cert_file, key_file, cred, gss_code);
+}
+
 /** Load or reload credentials. It should be called regularly (credential files can be changed).
-  This call works only for GSSAPI from Globus.
  @see edg_wll_gss_watch_creds
  */
 int
-edg_wll_gss_acquire_cred_gsi(const char *cert_file, const char *key_file, edg_wll_GssCred *cred,
-      			     edg_wll_GssStatus* gss_code)
+edg_wll_gss_acquire_cred(const char *cert_file, const char *key_file, edg_wll_GssCred *cred,
+			 edg_wll_GssStatus* gss_code)
 {
    OM_uint32 major_status = 0, minor_status, minor_status2;
    gss_cred_id_t gss_cred = GSS_C_NO_CREDENTIAL;
@@ -701,17 +711,21 @@ edg_wll_gss_acquire_cred_gsi(const char *cert_file, const char *key_file, edg_wl
    gss_OID_set avail_mechs = NULL;
    OM_uint32 lifetime;
    char *proxy_file = NULL;
-   char *name = NULL;
-   int ret, gsi_available;
+   char *name = NULL, *mech_name = NULL;
+   int ret, mech_available;
 
    *cred = NULL;
 
    major_status = gss_indicate_mechs(&minor_status, &avail_mechs);
    /* ignore error */
 
-   major_status = gss_test_oid_set_member(&minor_status, get_oid("GSI"),
-					  avail_mechs, &gsi_available);
-   if (!GSS_ERROR(major_status) && !gsi_available) {
+   mech_name = getenv("GLITE_GSS_MECH");
+   if (mech_name == NULL)
+	   mech_name = "GSI";
+
+   major_status = gss_test_oid_set_member(&minor_status, get_oid(mech_name),
+					  avail_mechs, &mech_available);
+   if (!GSS_ERROR(major_status) && !mech_available) {
        if (cert_file != NULL || key_file != NULL) {
 	   errno = EINVAL;
 	   ret = EDG_WLL_GSS_ERROR_ERRNO;
@@ -729,7 +743,7 @@ edg_wll_gss_acquire_cred_gsi(const char *cert_file, const char *key_file, edg_wl
 
    if (cert_file == NULL) {
       mechs.count = 1;
-      mechs.elements = get_oid("GSI");
+      mechs.elements = get_oid(mech_name);
       
       major_status = gss_acquire_cred(&minor_status, GSS_C_NO_NAME, 0,
 	    			      &mechs, GSS_C_BOTH,
@@ -738,7 +752,7 @@ edg_wll_gss_acquire_cred_gsi(const char *cert_file, const char *key_file, edg_wl
 	 ret = EDG_WLL_GSS_ERROR_GSS;
 	 goto end;
       }
-   } else {
+   } else if(!strcmp(mech_name, "GSI")) {
 #ifndef NO_GLOBUS_GSSAPI
       proxy_file = (char *)cert_file;
       if (strcmp(cert_file, key_file) != 0 &&
@@ -764,10 +778,16 @@ edg_wll_gss_acquire_cred_gsi(const char *cert_file, const char *key_file, edg_wl
 	 goto end;
       }
 #else
+      /* XXX - this mechanism is not supported */
       errno = EINVAL;
       ret = EDG_WLL_GSS_ERROR_ERRNO;
       goto end;
 #endif
+   } else {
+	   /* TODO: cert file specified for non-GSI mechanism - is it really error? */
+      errno = EINVAL;
+      ret = EDG_WLL_GSS_ERROR_ERRNO;
+      goto end;
    }
 
   /* gss_import_cred() doesn't check validity of credential loaded, so let's
@@ -838,6 +858,8 @@ end:
 
    return ret;
 }
+
+
 /* XXX XXX This is black magic. "Sometimes" server refuses the client with SSL
  * alert "certificate expired" even if it is not true. In this case the server
  * slave terminates (which helps, usually), and we can reconnect transparently.
@@ -1558,7 +1580,10 @@ edg_wll_gss_initialize(void)
 
    // some pre-initializations (workarounds thread-safe problem
    // in gss_init_sec_context)
+#ifndef NO_GLOBUS_GSSAPI
    globus_gsi_callback_get_SSL_callback_data_index(&index);
+#endif
+
 #ifdef GLITE_LBU_THREADED
    pthread_mutex_unlock(&init_lock);
 #endif
